package com.fjhx.area.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.area.entity.dto.AreaInfoSelectDto;
import com.fjhx.area.entity.po.AreaInfo;
import com.fjhx.area.mapper.AreaInfoMapper;
import com.fjhx.area.service.AreaInfoService;
import com.fjhx.area.service.ISetAreaName;
import com.ruoyi.common.constant.BaseSourceConstant;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * 国家城市表 服务实现类
 * </p>
 *
 * @author
 * @since 2023-03-17
 */
@Service
public class AreaInfoServiceImpl extends ServiceImpl<AreaInfoMapper, AreaInfo> implements AreaInfoService {

    @Override
    public List<AreaInfo> getList(AreaInfoSelectDto dto) {
        String parentId = ObjectUtil.defaultIfNull(dto.getParentId(), "0");
        return list(Wrappers.<AreaInfo>lambdaQuery().eq(AreaInfo::getParentId, parentId));
    }

    @DS(BaseSourceConstant.BASE)
    @Override
    public void setAreaName(List<? extends ISetAreaName> areaList) {
        Set<String> areaIdList = new HashSet<>();

        for (ISetAreaName iSetAreaName : areaList) {
            if (StrUtil.isNotBlank(iSetAreaName.getCountryId())) {
                areaIdList.add(iSetAreaName.getCountryId());
            }
            if (StrUtil.isNotBlank(iSetAreaName.getProvinceId())) {
                areaIdList.add(iSetAreaName.getProvinceId());
            }
            if (StrUtil.isNotBlank(iSetAreaName.getCityId())) {
                areaIdList.add(iSetAreaName.getCityId());
            }
        }

        if (areaIdList.size() == 0) {
            return;
        }

        // 地区
        List<AreaInfo> areaInfoList = listByIds(areaIdList);

        Map<String, AreaInfo> areaInfoMap = areaInfoList.stream().collect(Collectors.toMap(AreaInfo::getId, item -> item));

        for (ISetAreaName iSetAreaName : areaList) {
            if (StrUtil.isNotBlank(iSetAreaName.getCountryId())) {
                AreaInfo areaInfo = areaInfoMap.get(iSetAreaName.getCountryId());
                if (areaInfo != null) {
                    iSetAreaName.setCountryName(areaInfo.getName());
                }
            }
            if (StrUtil.isNotBlank(iSetAreaName.getProvinceId())) {
                AreaInfo areaInfo = areaInfoMap.get(iSetAreaName.getProvinceId());
                if (areaInfo != null) {
                    iSetAreaName.setProvinceName(areaInfo.getName());
                }
            }
            if (StrUtil.isNotBlank(iSetAreaName.getCityId())) {
                AreaInfo areaInfo = areaInfoMap.get(iSetAreaName.getCityId());
                if (areaInfo != null) {
                    iSetAreaName.setCityName(areaInfo.getName());
                }
            }
        }

    }

    @DS(BaseSourceConstant.BASE)
    @Override
    public Map<String, String> getAreaMapByIds(List<String> idList) {
        List<AreaInfo> areaInfos = listByIds(idList);
        return areaInfos.stream().collect(Collectors.toMap(AreaInfo::getId, AreaInfo::getName));
    }

}
