package com.fjhx.file.utils;

import cn.hutool.extra.spring.SpringUtil;
import com.fjhx.file.entity.FileInfoSelectDto;
import com.fjhx.file.entity.FileInfoVo;
import com.fjhx.file.entity.ObsFile;
import com.fjhx.file.service.FileInfoService;

import java.util.List;
import java.util.Map;

public class ObsFileUtil {

    private static final FileInfoService fileInfoService = SpringUtil.getBean(FileInfoService.class);
    private static final int defaultFileType = 0;

    /**
     * 保存文件
     *
     * @param obsFileList  文件列表
     * @param businessId   业务id
     * @param businessType 业务文件类型
     */
    public static void saveFile(List<ObsFile> obsFileList, Long businessId, Integer businessType) {
        fileInfoService.saveFile(obsFileList, businessId, businessType);
    }

    /**
     * 保存文件
     *
     * @param obsFileList 文件列表
     * @param businessId  业务id
     */
    public static void saveFile(List<ObsFile> obsFileList, Long businessId) {
        fileInfoService.saveFile(obsFileList, businessId, defaultFileType);
    }

    /**
     * 更新文件
     *
     * @param obsFileList  文件列表
     * @param businessId   业务id
     * @param businessType 业务文件类型
     */
    public static void editFile(List<ObsFile> obsFileList, Long businessId, Integer businessType) {
        fileInfoService.editFile(obsFileList, businessId, businessType);
    }

    /**
     * 更新文件
     *
     * @param obsFileList 文件列表
     * @param businessId  业务id
     */
    public static void editFile(List<ObsFile> obsFileList, Long businessId) {
        fileInfoService.editFile(obsFileList, businessId, defaultFileType);
    }

    /**
     * 删除文件
     *
     * @param businessId   业务id
     * @param businessType 业务文件类型
     */
    public static void removeFile(Long businessId, Integer businessType) {
        fileInfoService.removeFile(businessId, businessType);
    }

    /**
     * 删除文件
     *
     * @param businessId 业务id
     */
    public static void removeFile(Long businessId) {
        fileInfoService.removeFile(businessId);
    }

    public static Map<Long, List<FileInfoVo>> getFileMap(List<Long> businessIdList) {
        FileInfoSelectDto dto = new FileInfoSelectDto();
        dto.setBusinessIdList(businessIdList);
        return fileInfoService.getList(dto);
    }

    public static Map<Long, List<FileInfoVo>> getFileMap(List<Long> businessIdList, Integer fileType) {
        FileInfoSelectDto dto = new FileInfoSelectDto();
        dto.setBusinessIdList(businessIdList);
        dto.setFileType(fileType);
        return fileInfoService.getList(dto);
    }

}
