package com.fjhx.flow.core;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.utils.SecurityUtils;
import lombok.extern.slf4j.Slf4j;

import java.util.Date;
import java.util.Map;

@Slf4j
public abstract class FlowDelegate {

    public Map<String, Object> initTemplateMap(JSONObject currentData, JSONObject startData) {

        Map<String, Object> map = BeanUtil.beanToMap(currentData);
        map.put("startData", BeanUtil.beanToMap(startData));
        map.put("nickName", SecurityUtils.getLoginUser().getUser().getNickName());
        map.put("date", DateUtil.formatDate(new Date()));
        map.put("dateTime", DateUtil.formatDateTime(new Date()));

        return map;
    }

    /**
     * 获取流程key
     *
     * @return 流程key
     */
    public abstract String getFlowKey();

    /**
     * 开始流程实现方法
     *
     * @param flowId     流程id
     * @param submitData 流程入参
     * @return 业务id
     */
    public abstract Long start(Long flowId, JSONObject submitData);

    /**
     * 默认结束流程实现方法
     *
     * @param flowId     流程id
     * @param businessId 业务id
     * @param submitData 流程入参
     */
    public abstract void end(Long flowId, Long businessId, JSONObject submitData);

}
