package com.fjhx.flow.core;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.flow.enums.HandleTypeEnum;
import org.springframework.core.NamedThreadLocal;

import java.util.Map;

public class FlowThreadLocalUtil {

    private static final ThreadLocal<FlowThreadLocalUtil> FLOW_HOLDER = new NamedThreadLocal<>("hx-flow");

    private FlowThreadLocalUtil() {

    }

    /**
     * 流程id
     */
    private Long flowId;

    /**
     * 业务id
     */
    private Long businessId;

    /**
     * 当前节点提交参数
     */
    private JSONObject currentData;

    /**
     * 发起流程提交参数
     */
    private JSONObject startData;

    /**
     * 模板配置参数
     */
    private Map<String, Object> templateData;

    /**
     * 流程处理类型
     */
    private HandleTypeEnum handleTypeEnum;

    /**
     * 下个节点审批用户id
     */
    private Long nextHandleUserId;

    // ===================================================
    //  流程id
    // ===================================================
    public static Long getFlowId() {
        return getData().flowId;
    }

    public static void setFlowId(Long flowId) {
        getData().flowId = flowId;
    }

    // ===================================================
    //  业务id
    // ===================================================
    public static Long getBusinessId() {
        return getData().businessId;
    }

    public static void setBusinessId(Long businessId) {
        getData().businessId = businessId;
    }

    // ===================================================
    //  当前节点提交参数
    // ===================================================
    public static JSONObject getCurrentData() {
        return getData().currentData;
    }

    public static <T> T getCurrentData(Class<T> cls) {
        return getData().currentData.toJavaObject(cls);
    }

    public static void setCurrentData(JSONObject currentData) {
        getData().currentData = currentData;
    }

    // ===================================================
    //  发起流程提交参数
    // ===================================================
    public static JSONObject getStartData() {
        return getData().startData;
    }

    public static <T> T getStartData(Class<T> cls) {
        return getData().startData.toJavaObject(cls);
    }

    public static void setStartData(JSONObject startData) {
        getData().startData = startData;
    }

    // ===================================================
    //  模板配置参数
    // ===================================================
    public static Map<String, Object> getTemplateData() {
        return getData().templateData;
    }

    public static void setTemplateData(Map<String, Object> templateData) {
        getData().templateData = templateData;
    }

    // ===================================================
    //  流程处理类型
    // ===================================================
    public static HandleTypeEnum getHandleTypeEnum() {
        return getData().handleTypeEnum;
    }

    public static void setHandleTypeEnum(HandleTypeEnum handleTypeEnum) {
        getData().handleTypeEnum = handleTypeEnum;
    }

    // ===================================================
    //  下个节点审批用户id
    // ===================================================
    public static Long getNextHandleUserId() {
        return getData().nextHandleUserId;
    }

    public static void setNextHandleUserId(Long nextHandleUserId) {
        getData().nextHandleUserId = nextHandleUserId;
    }

    private static FlowThreadLocalUtil getData() {
        FlowThreadLocalUtil flowThreadLocalData = FLOW_HOLDER.get();
        if (flowThreadLocalData == null) {
            flowThreadLocalData = new FlowThreadLocalUtil();
            FLOW_HOLDER.set(flowThreadLocalData);
        }
        return flowThreadLocalData;
    }

}
