package com.fjhx.kd100.controller.callback;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.fjhx.kd100.entity.logistics.po.LogisticsInfo;
import com.fjhx.kd100.event.Kd100CallbackEvent;
import com.fjhx.kd100.service.logistics.LogisticsInfoService;
import com.fjhx.kd100.util.KD100Util;
import com.ruoyi.common.annotation.TenantIgnore;
import com.ruoyi.common.constant.BaseSourceConstant;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@DS(BaseSourceConstant.KD100)
@TenantIgnore
@RestController
@RequestMapping("/open/kd100")
public class CallbackConfig {

    @Autowired
    private LogisticsInfoService logisticsInfoService;

    @Autowired
    private ApplicationEventPublisher applicationEventPublisher;

    /**
     * 物流公司列表
     */
    @PostMapping("/callback")
    @DSTransactional
    public synchronized JSONObject callback(@RequestBody String data) {
        // 解析回调参数
        JSONObject result = KD100Util.parseCallbackData(data);

        handleCallback(result);

        JSONObject returnResult = new JSONObject();
        returnResult.put("result", true);
        returnResult.put("returnCode", 200);
        returnResult.put("message", "成功");
        return returnResult;
    }

    private void handleCallback(JSONObject result) {

        // 快递公司编码
        String com = result.getString("com");
        // 	单号
        String nu = result.getString("nu");
        // 物流状态
        Integer state = result.getInteger("state");

        List<LogisticsInfo> logisticsInfoList = logisticsInfoService.list(
                q -> q.eq(LogisticsInfo::getNumber, nu).eq(LogisticsInfo::getCompany, com));

        if (logisticsInfoList.size() == 0) {
            return;
        }
        
        for (LogisticsInfo logisticsInfo : logisticsInfoList) {
            logisticsInfo.setState(state);
            applicationEventPublisher.publishEvent(new Kd100CallbackEvent(this, logisticsInfo));
        }

        logisticsInfoService.updateBatchById(logisticsInfoList);
    }

}
