package com.fjhx.kd100.controller.config;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.kd100.entity.config.dto.ConfigInfoDto;
import com.fjhx.kd100.entity.config.dto.ConfigInfoSelectDto;
import com.fjhx.kd100.entity.config.po.ConfigInfo;
import com.fjhx.kd100.entity.config.vo.ConfigInfoVo;
import com.fjhx.kd100.service.config.ConfigInfoService;
import com.ruoyi.common.annotation.TenantIgnore;
import com.ruoyi.common.constant.BaseSourceConstant;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * <p>
 * 租户快递100账户配置 前端控制器
 * </p>
 *
 * @author
 * @since 2023-04-11
 */
@DS(BaseSourceConstant.KD100)
@TenantIgnore
@RestController
@RequestMapping("/configInfo")
public class ConfigInfoController {

    @Autowired
    private ConfigInfoService configInfoService;

    /**
     * 租户快递100账户配置分页
     */
    @PostMapping("/page")
    public Page<ConfigInfoVo> page(@RequestBody ConfigInfoSelectDto dto) {
        return configInfoService.getPage(dto);
    }

    /**
     * 租户快递100账户配置明细
     */
    @PostMapping("/detail")
    public ConfigInfoVo detail(@RequestBody BaseSelectDto dto) {
        return configInfoService.detail(dto.getId());
    }

    /**
     * 租户快递100账户配置新增
     */
    @PostMapping("/add")
    public void add(@Validated @RequestBody ConfigInfoDto configInfoDto) {
        configInfoService.add(configInfoDto);
    }

    /**
     * 租户快递100账户配置编辑
     */
    @PostMapping("/edit")
    public void edit(@RequestBody ConfigInfoDto configInfoDto) {
        configInfoService.edit(configInfoDto);
    }

    /**
     * 租户快递100账户配置删除
     */
    @PostMapping("/delete")
    public void delete(@RequestBody BaseSelectDto dto) {
        configInfoService.delete(dto.getId());
    }

    /**
     * 获取当前租户快递100配置
     */
    @PostMapping("/getMyKd100Config")
    public ConfigInfo getMyKd100Config() {
        return configInfoService.getOne(q -> q.eq(ConfigInfo::getTenantId, SecurityUtils.getTenantId()));
    }

}
