package com.fjhx.tenant.controller.tenant;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.ruoyi.common.annotation.TenantIgnore;
import com.ruoyi.common.constant.BaseSourceConstant;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@TenantIgnore
@DS(BaseSourceConstant.BASE)
@RestController
@RequestMapping("/tenantDept")
public class DetpTenantController {

    @Autowired
    private ISysDeptService deptService;

    /**
     * 获取部门列表
     */
    @GetMapping("/list")
    public List<SysDept> list(SysDept dept) {
        return deptService.selectDeptList(dept);
    }

    /**
     * 新增部门
     */
    @PostMapping
    public void add(@Validated @RequestBody SysDept dept) {
        String tenantId = dept.getTenantId();
        if (ObjectUtil.isEmpty(tenantId)) {
            throw new ServiceException("租户id不能为空");
        }

        if (!deptService.checkDeptNameUnique(dept)) {
            throw new ServiceException("新增部门'" + dept.getDeptName() + "'失败，部门名称已存在");
        }
        dept.setCreateBy(SecurityUtils.getUsername());
        deptService.insertDept(dept);
    }

    /**
     * 修改部门
     */
    @PutMapping
    public void edit(@Validated @RequestBody SysDept dept) {
        String tenantId = dept.getTenantId();
        if (ObjectUtil.isEmpty(tenantId)) {
            throw new ServiceException("租户id不能为空");
        }

        Long deptId = dept.getDeptId();
        deptService.checkDeptDataScope(deptId);
        if (!deptService.checkDeptNameUnique(dept)) {
            throw new ServiceException("修改部门'" + dept.getDeptName() + "'失败，部门名称已存在");
        } else if (dept.getParentId().equals(deptId)) {
            throw new ServiceException("修改部门'" + dept.getDeptName() + "'失败，上级部门不能是自己");
        } else if (StringUtils.equals(UserConstants.DEPT_DISABLE, dept.getStatus()) && deptService.selectNormalChildrenDeptById(deptId) > 0) {
            throw new ServiceException("该部门包含未停用的子部门！");
        }
        dept.setUpdateBy(SecurityUtils.getUsername());
        deptService.updateDept(dept);
    }

    /**
     * 删除部门
     */
    @DeleteMapping("/{deptId}")
    public void remove(@PathVariable Long deptId) {
        if (deptService.hasChildByDeptId(deptId)) {
            throw new ServiceException("存在下级部门,不允许删除");
        }
        if (deptService.checkDeptExistUser(deptId)) {
            throw new ServiceException("部门存在用户,不允许删除");
        }
        deptService.checkDeptDataScope(deptId);
        deptService.deleteDeptById(deptId);
    }

}
