package com.fjhx.tenant.controller.tenant;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.file.entity.FileInfoVo;
import com.fjhx.tenant.entity.tenant.dto.BindingMenuDto;
import com.fjhx.tenant.entity.tenant.dto.TenantInfoDto;
import com.fjhx.tenant.entity.tenant.dto.TenantInfoSelectDto;
import com.fjhx.tenant.entity.tenant.po.TenantInfo;
import com.fjhx.tenant.entity.tenant.vo.TenantInfoVo;
import com.fjhx.tenant.service.tenant.TenantInfoService;
import com.ruoyi.common.annotation.TenantIgnore;
import com.ruoyi.common.constant.BaseSourceConstant;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.core.domain.entity.SysMenu;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.service.ISysMenuService;
import com.ruoyi.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * <p>
 * 租户表 前端控制器
 * </p>
 *
 * @author zlj
 * @since 2023-03-15
 */
@TenantIgnore
@DS(BaseSourceConstant.BASE)
@RestController
@RequestMapping("/tenantInfo")
public class TenantInfoController {

    @Autowired
    private TenantInfoService tenantInfoService;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private ISysRoleService sysRoleService;

    /**
     * 租户表分页
     */
    @PostMapping("/list")
    public List<TenantInfo> list(@RequestBody TenantInfoSelectDto dto) {
        return tenantInfoService.getList(dto);
    }

    /**
     * 租户表分页
     */
    @PostMapping("/page")
    public Page<TenantInfoVo> page(@RequestBody TenantInfoSelectDto dto) {
        return tenantInfoService.getPage(dto);
    }

    /**
     * 租户表明细
     */
    @PostMapping("/detail")
    public TenantInfoVo detail(@RequestBody BaseSelectDto dto) {
        return tenantInfoService.detail(dto.getId());
    }

    /**
     * 租户表新增
     */
    @PostMapping("/add")
    public void add(@RequestBody TenantInfoDto tenantInfoDto) {
        tenantInfoService.add(tenantInfoDto);
    }

    /**
     * 租户表编辑
     */
    @PostMapping("/edit")
    public void edit(@RequestBody TenantInfoDto tenantInfoDto) {
        tenantInfoService.edit(tenantInfoDto);
    }

    /**
     * 租户表删除
     */
    @PostMapping("/delete")
    public void delete(@RequestBody BaseSelectDto dto) {
        tenantInfoService.delete(dto.getId());
    }

    /**
     * 租户绑定菜单
     */
    @PostMapping("/bindingMenu")
    public void bindingMenu(@RequestBody @Validated BindingMenuDto dto) {
        tenantInfoService.bindingMenu(dto);
    }

    /**
     * 加载对应角色菜单列表树
     */
    @GetMapping(value = "/roleMenuTreeSelect/{tenantId}")
    public AjaxResult roleMenuTreeSelect(@PathVariable String tenantId) {

        SysRole sysRole = sysRoleService.getOne(Wrappers.<SysRole>lambdaQuery()
                .eq(SysRole::getTenantId, tenantId).eq(SysRole::getRoleKey, "admin"));

        List<SysMenu> menus = menuService.selectMenuList(SecurityUtils.getUserId());
        AjaxResult ajax = AjaxResult.success();
        ajax.put("checkedKeys", menuService.selectMenuListByRoleId(sysRole.getRoleId()));
        ajax.put("menus", menuService.buildMenuTreeSelect(menus));
        return ajax;
    }

    /**
     * 编辑logo
     */
    @PostMapping("/editLogo")
    public void editLogo(@RequestBody TenantInfoDto tenantInfoDto) {
        tenantInfoService.editLogo(tenantInfoDto);
    }

    /**
     * 获取logo
     */
    @PostMapping("/getLogo")
    public List<FileInfoVo> getLogo() {
        return tenantInfoService.getLogo();
    }


}
