package com.fjhx.tenant.service.dict.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.tenant.entity.dict.dto.DictCommonDataDto;
import com.fjhx.tenant.entity.dict.dto.DictCommonDataSelectDto;
import com.fjhx.tenant.entity.dict.po.DictCommonData;
import com.fjhx.tenant.entity.dict.po.DictTenantData;
import com.fjhx.tenant.entity.dict.vo.DictCommonDataVo;
import com.fjhx.tenant.mapper.dict.DictCommonDataMapper;
import com.fjhx.tenant.service.dict.DictCommonDataService;
import com.fjhx.tenant.service.dict.DictTenantDataService;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Objects;


/**
 * <p>
 * 业务字典明细 服务实现类
 * </p>
 *
 * @author
 * @since 2023-03-30
 */
@Service
public class DictCommonDataServiceImpl extends ServiceImpl<DictCommonDataMapper, DictCommonData> implements DictCommonDataService {

    @Autowired
    private DictTenantDataService dictTenantDataService;

    @Override
    public Page<DictCommonDataVo> getPage(DictCommonDataSelectDto dto) {
        IWrapper<DictCommonData> wrapper = getWrapper();
        wrapper.eq("dcd", DictCommonData::getDictCode, dto.getDictCode());
        wrapper.orderByAsc("dcd", DictCommonData::getSort);
        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    @Override
    public DictCommonDataVo detail(Long id) {
        DictCommonData DictCommonData = this.getById(id);
        DictCommonDataVo result = BeanUtil.toBean(DictCommonData, DictCommonDataVo.class);
        return result;
    }

    @Override
    public void add(DictCommonDataDto dictCommonDataDto) {

        long dictTenantDataKeyCount = dictTenantDataService.count(q -> q
                .eq(DictTenantData::getDictCode, dictCommonDataDto.getDictCode())
                .eq(DictTenantData::getDictKey, dictCommonDataDto.getDictKey()));
        if (dictTenantDataKeyCount > 0) {
            throw new ServiceException("租户字典key已存在");
        }

        long dictTenantDataValueCount = dictTenantDataService.count(q -> q
                .eq(DictTenantData::getDictCode, dictCommonDataDto.getDictCode())
                .eq(DictTenantData::getDictValue, dictCommonDataDto.getDictValue()));
        if (dictTenantDataValueCount > 0) {
            throw new ServiceException("租户字典value已存在");
        }

        long dictCommonDataKeyCount = count(q -> q
                .eq(DictCommonData::getDictCode, dictCommonDataDto.getDictCode())
                .eq(DictCommonData::getDictKey, dictCommonDataDto.getDictKey()));
        if (dictCommonDataKeyCount > 0) {
            throw new ServiceException("业务字典key已存在");
        }

        long dictCommonDataValueCount = count(q -> q
                .eq(DictCommonData::getDictCode, dictCommonDataDto.getDictCode())
                .eq(DictCommonData::getDictValue, dictCommonDataDto.getDictValue()));
        if (dictCommonDataValueCount > 0) {
            throw new ServiceException("业务字典value已存在");
        }

        this.save(dictCommonDataDto);
    }

    @Override
    public void edit(DictCommonDataDto dictCommonDataDto) {
        dictCommonDataDto.setDictCode(null);

        Long id = dictCommonDataDto.getId();
        if (Objects.isNull(id)) {
            throw new ServiceException("id不能为空");
        }

        DictCommonData dictCommonData = getById(id);

        long dictTenantDataKeyCount = dictTenantDataService.count(q -> q
                .eq(DictTenantData::getDictCode, dictCommonData.getDictCode())
                .eq(DictTenantData::getDictKey, dictCommonDataDto.getDictKey()));
        if (dictTenantDataKeyCount > 0) {
            throw new ServiceException("租户字典key已存在");
        }

        long dictTenantDataValueCount = dictTenantDataService.count(q -> q
                .eq(DictTenantData::getDictCode, dictCommonData.getDictCode())
                .eq(DictTenantData::getDictValue, dictCommonDataDto.getDictValue()));
        if (dictTenantDataValueCount > 0) {
            throw new ServiceException("租户字典value已存在");
        }

        long dictCommonDataKeyCount = count(q -> q
                .eq(DictCommonData::getDictCode, dictCommonData.getDictCode())
                .eq(DictCommonData::getDictKey, dictCommonDataDto.getDictKey())
                .ne(BaseIdPo::getId, id));
        if (dictCommonDataKeyCount > 0) {
            throw new ServiceException("业务字典key已存在");
        }

        long dictCommonDataValueCount = count(q -> q
                .eq(DictCommonData::getDictCode, dictCommonData.getDictCode())
                .eq(DictCommonData::getDictValue, dictCommonDataDto.getDictValue())
                .ne(BaseIdPo::getId, id));
        if (dictCommonDataValueCount > 0) {
            throw new ServiceException("业务字典value已存在");
        }

        this.updateById(dictCommonDataDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
