package com.fjhx.tenant.service.dict.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.tenant.entity.dict.dto.DictTenantDataDto;
import com.fjhx.tenant.entity.dict.dto.DictTenantDataSelectDto;
import com.fjhx.tenant.entity.dict.po.DictCommonData;
import com.fjhx.tenant.entity.dict.po.DictTenantData;
import com.fjhx.tenant.entity.dict.vo.DictTenantDataVo;
import com.fjhx.tenant.mapper.dict.DictTenantDataMapper;
import com.fjhx.tenant.service.dict.DictCommonDataService;
import com.fjhx.tenant.service.dict.DictTenantDataService;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;


/**
 * <p>
 * 租户字典明细 服务实现类
 * </p>
 *
 * @author
 * @since 2023-03-30
 */
@Service
public class DictTenantDataServiceImpl extends ServiceImpl<DictTenantDataMapper, DictTenantData> implements DictTenantDataService {

    @Autowired
    private DictCommonDataService dictCommonDataService;

    @Override
    public Page<DictTenantDataVo> getPage(DictTenantDataSelectDto dto) {
        return this.baseMapper.getPage(dto.getPage(), dto);
    }

    @Override
    public DictTenantDataVo detail(Long id) {
        DictTenantData DictTenantData = this.getById(id);
        DictTenantDataVo result = BeanUtil.toBean(DictTenantData, DictTenantDataVo.class);
        return result;
    }

    @Override
    public void add(DictTenantDataDto dictTenantDataDto) {

        long dictTenantDataKeyCount = count(q -> q
                .eq(DictTenantData::getDictCode, dictTenantDataDto.getDictCode())
                .eq(DictTenantData::getDictKey, dictTenantDataDto.getDictKey())
                .eq(DictTenantData::getTenantId, dictTenantDataDto.getTenantId()));
        if (dictTenantDataKeyCount > 0) {
            throw new ServiceException("租户字典key已存在");
        }

        long dictTenantDataValueCount = count(q -> q
                .eq(DictTenantData::getDictCode, dictTenantDataDto.getDictCode())
                .eq(DictTenantData::getDictValue, dictTenantDataDto.getDictValue())
                .eq(DictTenantData::getTenantId, dictTenantDataDto.getTenantId()));
        if (dictTenantDataValueCount > 0) {
            throw new ServiceException("租户字典value已存在");
        }

        long dictCommonDataKeyCount = dictCommonDataService.count(q -> q
                .eq(DictCommonData::getDictCode, dictTenantDataDto.getDictCode())
                .eq(DictCommonData::getDictKey, dictTenantDataDto.getDictKey()));
        if (dictCommonDataKeyCount > 0) {
            throw new ServiceException("业务字典key已存在");
        }

        long dictCommonDataValueCount = dictCommonDataService.count(q -> q
                .eq(DictCommonData::getDictCode, dictTenantDataDto.getDictCode())
                .eq(DictCommonData::getDictValue, dictTenantDataDto.getDictValue()));
        if (dictCommonDataValueCount > 0) {
            throw new ServiceException("业务字典value已存在");
        }

        this.save(dictTenantDataDto);
    }

    @Override
    public void edit(DictTenantDataDto dictTenantDataDto) {

        Long id = dictTenantDataDto.getId();
        if (Objects.isNull(id)) {
            throw new ServiceException("id不能为空");
        }

        dictTenantDataDto.setDictCode(null);

        DictTenantData dictTenantData = getById(id);

        long dictTenantDataKeyCount = count(q -> q
                .eq(DictTenantData::getDictCode, dictTenantData.getDictCode())
                .eq(DictTenantData::getDictKey, dictTenantDataDto.getDictKey())
                .eq(DictTenantData::getTenantId, dictTenantDataDto.getTenantId())
                .ne(BaseIdPo::getId, id)
        );
        if (dictTenantDataKeyCount > 0) {
            throw new ServiceException("租户字典key已存在");
        }

        long dictTenantDataValueCount = count(q -> q
                .eq(DictTenantData::getDictCode, dictTenantData.getDictCode())
                .eq(DictTenantData::getDictValue, dictTenantDataDto.getDictValue())
                .eq(DictTenantData::getTenantId, dictTenantDataDto.getTenantId())
                .ne(BaseIdPo::getId, id)
        );
        if (dictTenantDataValueCount > 0) {
            throw new ServiceException("租户字典value已存在");
        }

        long dictCommonDataKeyCount = dictCommonDataService.count(q -> q
                .eq(DictCommonData::getDictCode, dictTenantData.getDictCode())
                .eq(DictCommonData::getDictKey, dictTenantDataDto.getDictKey()));
        if (dictCommonDataKeyCount > 0) {
            throw new ServiceException("业务字典key已存在");
        }

        long dictCommonDataValueCount = dictCommonDataService.count(q -> q
                .eq(DictCommonData::getDictCode, dictTenantData.getDictCode())
                .eq(DictCommonData::getDictValue, dictTenantDataDto.getDictValue()));
        if (dictCommonDataValueCount > 0) {
            throw new ServiceException("业务字典value已存在");
        }

        this.updateById(dictTenantDataDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public List<DictTenantDataVo> getList(DictTenantDataSelectDto dto) {
        String dictCode = dto.getDictCode();
        if (StrUtil.isBlank(dictCode)) {
            throw new ServiceException("字典编码不能为空");
        }

        dto.setTenantId(SecurityUtils.getTenantId());
        return baseMapper.getList(dto);
    }

}
