package com.ruoyi.common.core.service;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.AttributeAssignBuilder;
import com.ruoyi.common.utils.wrapper.IWrapper;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.stream.Collectors;

@SuppressWarnings({"unused", "unchecked"})
public interface BaseService<T extends BaseIdPo> extends IService<T> {

    default <V> AttributeAssignBuilder<T, V> attributeAssignBuilder(List<V> list) {
        return AttributeAssignBuilder.init(this, list);
    }

    default <V> void attributeAssign(List<V> list, Function<V, Long> getIdFun, BiConsumer<V, T> setAttributeFun) {
        attributeAssignBuilder(list).assignFun(getIdFun, setAttributeFun).build();
    }

    default IWrapper<T> getWrapper() {
        return IWrapper.getWrapper();
    }

    // =========================================================
    // mp方法扩展
    // =========================================================

    default void assertExecutable(Consumer<LambdaQueryWrapper<T>> consumer, String errorMsgTemplate, Object... params) {
        long count = count(consumer);
        Assert.equals(count, 0L, errorMsgTemplate, params);
    }

    default long count(Consumer<LambdaQueryWrapper<T>> consumer) {
        return count(Wrappers.<T>lambdaQuery().func(ObjectUtil.isNotEmpty(consumer), consumer));
    }

    default T getOne(Consumer<LambdaQueryWrapper<T>> consumer) {
        return getOne(Wrappers.<T>lambdaQuery().func(ObjectUtil.isNotEmpty(consumer), consumer).last("limit 1"));
    }

    default List<T> list(Consumer<LambdaQueryWrapper<T>> consumer) {
        return list(Wrappers.<T>lambdaQuery().func(ObjectUtil.isNotEmpty(consumer), consumer));
    }

    default List<Map<String, Object>> listMaps(Consumer<LambdaQueryWrapper<T>> consumer) {
        return listMaps(Wrappers.<T>lambdaQuery().func(ObjectUtil.isNotEmpty(consumer), consumer));
    }

    default <K> List<K> listObject(SFunction<T, K> mapper, Consumer<LambdaQueryWrapper<T>> consumer) {
        List<T> list = list(Wrappers.<T>lambdaQuery().select(mapper).func(ObjectUtil.isNotEmpty(consumer), consumer));
        return list.stream().map(mapper).filter(Objects::nonNull).distinct().collect(Collectors.toList());
    }

    default <K, V> Map<K, V> mapKV(SFunction<T, K> kFun, SFunction<T, V> vFun, Consumer<LambdaQueryWrapper<T>> consumer) {
        List<T> list = list(Wrappers.<T>lambdaQuery().select(kFun, vFun).func(ObjectUtil.isNotEmpty(consumer), consumer));
        return list.stream().collect(Collectors.toMap(kFun, vFun, (v1, v2) -> v2));
    }

    default <K> Map<K, T> mapKEntity(SFunction<T, K> kFun, Consumer<LambdaQueryWrapper<T>> consumer) {
        List<T> list = list(Wrappers.<T>lambdaQuery().func(ObjectUtil.isNotEmpty(consumer), consumer));
        return list.stream().collect(Collectors.toMap(kFun, item -> item, (v1, v2) -> v2));
    }

    default <K> Map<K, List<T>> mapKGroup(SFunction<T, K> kFun, Consumer<LambdaQueryWrapper<T>> consumer) {
        List<T> list = list(Wrappers.<T>lambdaQuery().func(ObjectUtil.isNotEmpty(consumer), consumer));
        return list.stream().collect(Collectors.groupingBy(kFun));
    }

    default boolean update(Consumer<LambdaUpdateWrapper<T>> consumer) {
        return update(Wrappers.<T>lambdaUpdate().func(ObjectUtil.isNotEmpty(consumer), consumer));
    }

    default boolean remove(Consumer<LambdaQueryWrapper<T>> consumer) {
        return remove(Wrappers.<T>lambdaQuery().func(ObjectUtil.isNotEmpty(consumer), consumer));
    }

    default void nameDuplication(SFunction<T, ?> column, Object value, String errMsg) {
        long count = count(q -> q.eq(column, value));
        if (count != 0) {
            throw new ServiceException(errMsg);
        }
    }

    default void nameDuplication(SFunction<T, ?> column, Object value, Long id, String errMsg) {
        long count = count(q -> q.eq(column, value).ne(T::getId, id));
        if (count != 0) {
            throw new ServiceException(errMsg);
        }
    }

    /**
     * 获取添加过的某个字段的去重列表
     */
    default <V> List<V> getDistinctList(SFunction<T, V> column) {
        return list(Wrappers.<T>lambdaQuery().select(column))
                .stream().map(column).filter(Objects::nonNull).distinct().collect(Collectors.toList());
    }

    /**
     * 编辑关联表
     */
    default void editLinked(List<T> list, SFunction<T, Long> getMasterIdFun, Long masterId) {
        List<Long> idList = list.stream().map(BaseIdPo::getId).filter(Objects::nonNull).collect(Collectors.toList());
        remove(q -> q.eq(getMasterIdFun, masterId).notIn(ObjectUtil.isNotEmpty(idList), BaseIdPo::getId, idList));
        saveOrUpdateBatch(list);
    }

    /**
     * 分页
     */
    default Page<T> page(Page<T> page, Consumer<LambdaQueryWrapper<T>> consumer) {
        return page(page, Wrappers.<T>lambdaQuery().func(ObjectUtil.isNotEmpty(consumer), consumer));
    }


}
