package com.ruoyi.framework.config;

import com.ruoyi.common.annotation.NonInterception;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.page.TableDataInfo;
import org.springframework.core.MethodParameter;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

import java.lang.reflect.AnnotatedElement;
import java.util.LinkedHashMap;
import java.util.Objects;

/**
 * 做统一格式返回
 */
@ControllerAdvice
public class ResponseAdvice implements ResponseBodyAdvice<Object> {

    @Override
    public boolean supports(MethodParameter returnType, Class<? extends HttpMessageConverter<?>> converterType) {
        AnnotatedElement annotatedElement = returnType.getAnnotatedElement();
        NonInterception annotation = annotatedElement.getAnnotation(NonInterception.class);
        return annotation == null;
    }

    @Override
    public Object beforeBodyWrite(Object body, MethodParameter returnType, MediaType selectedContentType, Class<? extends HttpMessageConverter<?>> selectedConverterType, ServerHttpRequest request, ServerHttpResponse response) {
        // 404处理
        if (body instanceof LinkedHashMap) {
            Object status = ((LinkedHashMap<?, ?>) body).get("status");
            Object error = ((LinkedHashMap<?, ?>) body).get("error");
            if (Objects.equals(status, 404) && Objects.equals(error, "Not Found")) {
                return R.fail(404, "Not Found");
            }
            if (Objects.equals(status, 500) && Objects.equals(error, "Internal Server Error")) {
                return R.fail(500, "Internal Server Error");
            }
        }

        // 提供一定的灵活度，如果body已经被包装了，就不进行包装
        if (body instanceof R || body instanceof AjaxResult || body instanceof TableDataInfo) {
            return body;
        }
        return R.ok(body);
    }

}