package com.ruoyi.framework.mybatis.interceptor;

import cn.hutool.core.util.StrUtil;
import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.stat.JdbcDataSourceStat;
import com.baomidou.dynamic.datasource.DynamicRoutingDataSource;
import com.baomidou.dynamic.datasource.ds.ItemDataSource;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.extension.plugins.handler.TenantLineHandler;
import com.baomidou.mybatisplus.extension.plugins.inner.TenantLineInnerInterceptor;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.framework.mybatis.holder.TenantHolder;
import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.expression.StringValue;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;

import javax.sql.DataSource;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Component
public class TenantInterceptor extends TenantLineInnerInterceptor {

    private static final String TenantName = "tenant_id";

    // 包涵租户字段集合
    private static final Map<String, List<String>> includeTenantIdTableNameMap = new ConcurrentHashMap<>();

    public TenantInterceptor(DataSource dataSource) {
        DynamicRoutingDataSource dynamicRoutingDataSource = (DynamicRoutingDataSource) dataSource;
        Map<String, DataSource> dataSources = dynamicRoutingDataSource.getDataSources();
        dataSources.forEach((k, v) -> {
            // 获取dataSourceStat
            JdbcDataSourceStat dataSourceStat = ((DruidDataSource) ((ItemDataSource) v).getRealDataSource()).getDataSourceStat();
            // 获取链接url
            String url = dataSourceStat.getUrl();
            // 获取数据库名
            String dbName = url.split("/")[3].split("\\?")[0];
            // 查询包含逻辑删除字段的表名sql
            String sql = "SELECT table_name FROM information_schema.COLUMNS WHERE table_schema = ? AND column_name = ?";
            // 执行sql找出不包含逻辑删除字段的表名
            List<String> includeTenantIdTableNameList = new JdbcTemplate(v).queryForList(sql, String.class, dbName, TenantName);
            // 数据赋值
            includeTenantIdTableNameMap.put(dataSourceStat.getName(), includeTenantIdTableNameList);
        });

        setTenantLineHandler(new TenantLineHandler() {

            @Override
            public Expression getTenantId() {
                return new StringValue(SecurityUtils.getTenantId());
            }

            @Override
            public String getTenantIdColumn() {
                return TenantName;
            }

            @Override
            public boolean ignoreTable(String tableName) {

                // 存在忽略自动拼接租户注解，不拼接租户字段
                if (TenantHolder.isIgnore()) {
                    return true;
                }

                // 获取当前线程处理的数据源类型
                String dataSourceType = DynamicDataSourceContextHolder.peek();
                if (StrUtil.isBlank(dataSourceType)) {
                    throw new ServiceException("未指定数据源");
                }

                // 获取当前数据原中包涵租户字段的表名
                List<String> tableNameList = includeTenantIdTableNameMap.get(dataSourceType);
                if (tableNameList == null) {
                    throw new ServiceException("未找到DS数据源");
                }

                return !tableNameList.contains(tableName);
            }

        });
    }

}
